/**
 * <copyright>
 *
 * Copyright (c) 2004 Chaur G. Wu. All rights reserved. 
 *
 * This program and the accompanying materials are made available 
 * under the terms of the Common Public License v1.0 which accompanies 
 * this distribution, and is available at 
 * http://www.eclipse.org/legal/cpl-v10.html
 *
 * </copyright>
 */

package com.example.rule.ecore.templates;

import org.eclipse.emf.common.util.*;
import com.example.rule.ecore.rulemetamodel.*;
import java.util.Iterator;
import java.util.Hashtable;

/**
 * @author Gwu
 *
 * TODO To change the template for this generated type comment go to
 * Window - Preferences - Java - Code Style - Code Templates
 */
public class ModuleHelper {
	
	public String factTemplateToListString(RFactTemplate factTemplate) {
		StringBuffer stringBuffer = new StringBuffer();
		stringBuffer.append("(deftemplate " + factTemplate.getFactName().toListString());
		for (Iterator si = factTemplate.getSlot().iterator(); si.hasNext(); ) {
			RFactSlot factSlot = (RFactSlot) si.next();
			stringBuffer.append("\n (slot " + factSlot.getSlotName().toListString() + ")");
		}
		stringBuffer.append(")");
		return stringBuffer.toString();
	}
	
	public String getMatchedFactListString(RRule rule)	{
		StringBuffer stringBuffer = new StringBuffer("\n  ");
		Hashtable tbl = new Hashtable();
		EList patterns = rule.getCeMatchPattern();
		RCondElem ce = rule.getCondition();
		
		checkFactTemplateInCe(ce, tbl);
		for (Iterator pi = patterns.iterator(); pi.hasNext(); ) {
		  	  RPattern pattern = (RPattern) pi.next();
		  	  if (!tbl.contains(pattern.getFactTemplate().getFactName())) {
		  	  	tbl.put(pattern.getFactTemplate().getFactName(), pattern.getFactTemplate());
		  	  }
		}
		
		Object[] factTemplates = tbl.values().toArray();
		for (int i = 0; i < factTemplates.length; i++) {
			RFactTemplate factTemplate = (RFactTemplate) factTemplates[i];
			stringBuffer.append("?" + factTemplate.getFactName().toListString() + " <- (" + factTemplate.getFactName().toListString());
			for (Iterator si = factTemplate.getSlot().iterator(); si.hasNext(); )	{
				RFactSlot factSlot = (RFactSlot) si.next();
				String slotName = factSlot.getSlotName().toListString();
				stringBuffer.append(" (" + slotName + " ?" + slotName + ")");
			}
			stringBuffer.append(")");
		}
		
		return stringBuffer.toString();
	}
	
	private void checkFactTemplateInCe(RCondElem ce, Hashtable tbl) {
		if (ce == null) return;
		if (tbl == null) return;
		
		EList childCEs = ce.getChildCE();
		for (Iterator cei = childCEs.iterator(); cei.hasNext(); ) {
			RCondElem childCE = (RCondElem) cei.next();
			checkFactTemplateInCe(childCE, tbl);
		}
		
		EList cePatterns = ce.getCeMatchPattern();
		for (Iterator pi = cePatterns.iterator(); pi.hasNext(); ) {
		  	  RPattern pattern = (RPattern) pi.next();
		  	  if (!tbl.contains(pattern.getFactTemplate().getFactName())) {
		  	  	tbl.put(pattern.getFactTemplate().getFactName(), pattern.getFactTemplate());
		  	  }
		}
	}
	
	public String ceToListString(RCondElem ce, RRuleType ruleType) {
		if (ce == null) return "";
		
		StringBuffer stringBuffer = new StringBuffer("\n  ");
		stringBuffer.append("(" + ce.getCeName().toString());
		
		EList childCEs = ce.getChildCE();
		for (Iterator cei = childCEs.iterator(); cei.hasNext(); ) {
			RCondElem childCE = (RCondElem) cei.next();
			String childCEListString = this.ceToListString(childCE, ruleType);
			stringBuffer.append(childCEListString);
		}
		
		EList cePatterns = ce.getCeMatchPattern();
		for (Iterator pi = cePatterns.iterator(); pi.hasNext(); ) {
		  	  RPattern pattern = (RPattern) pi.next();
		  	  stringBuffer.append("\n\t" + patternToListString(pattern, ruleType));
		}
		
		/*?matchedFact <- (placement (box_one ?box_one) (box_two ?box_two) (box_three ?box_three) (box_four ?box_four))
				  (and (placement (box_one ?box_one&Red) (box_two ?box_two) (box_three ?box_three) (box_four ?box_four))
					   (placement (box_one ?box_one) (box_two ?box_two) (box_three ?box_three&~Blue) (box_four ?box_four))
				  )
		*/
		stringBuffer.append(")");
		return stringBuffer.toString();		  
	}

	/**
	 * The string returned will look something like this:
	 *(placement (box_one ?box_one&Red) (box_two ?box_two) (box_three ?box_three) (box_four ?box_four))
	 */
	public String patternToListString(RPattern pattern, RRuleType ruleType) {
		StringBuffer stringBuffer = new StringBuffer();
		RFactTemplate factTemplate = pattern.getFactTemplate();
		
		//Put all the pattern's match expressions in a hashtable 
		//so that we can look them up later.
		EList matchExps = pattern.getSlotMatchExp();
		Hashtable tbl = new Hashtable();
		for (Iterator expi = matchExps.iterator(); expi.hasNext(); ) {
			RSlotMatchExp exp = (RSlotMatchExp) expi.next();
			tbl.put(exp.getSlotName(), exp.getMatchExp());
		}
		
		String matchExpPrefix = "";
		if (ruleType == RRuleType.CONSTRAINT_LITERAL) {
			matchExpPrefix = "&";
		}
			
		stringBuffer.append("  (" + factTemplate.getFactName().toListString());
		for (Iterator si = factTemplate.getSlot().iterator(); si.hasNext(); )	{
			RFactSlot factSlot = (RFactSlot) si.next();
			String matchExp = ""; 
			if (tbl.containsKey(factSlot.getSlotName())) {
				matchExp = matchExpPrefix + (String) tbl.get(factSlot.getSlotName());
			}
			String slotName = factSlot.getSlotName().toListString();
			stringBuffer.append(" (" + slotName + " ");
			if (ruleType == RRuleType.CONSTRAINT_LITERAL)
				stringBuffer.append("?" + slotName);
			
			stringBuffer.append(matchExp + ")");
		}
		
		stringBuffer.append(")");
		return stringBuffer.toString();		  
	}
	
	public String actionToListString(RFunction func) {
		StringBuffer stringBuffer = new StringBuffer();
		stringBuffer.append("(" + func.getFunctionName().toListString());
		EList params = func.getParameters();
		for (Iterator pi = params.iterator(); pi.hasNext(); )	{
			RDataType param = (RDataType) pi.next();
			stringBuffer.append(" " + param.toListString());
		}
		
		stringBuffer.append(")");
		return stringBuffer.toString();		 
	}
}
